@@ -153,7 +153,7 @@ fn format_iso(cls: PyType, slf: YearMonth) -> PyReturn {
 
 fn parse_iso(cls: HeapType<YearMonth>, arg: PyObj) -> PyReturn {
     let py_str = arg
-        .cast::<PyStr>()
+        .cast_allow_subclass::<PyStr>()
         // NOTE: this exception message also needs to make sense when
         // called through the constructor
         .ok_or_type_err("When parsing from ISO format, the argument must be str")?;
@@ -200,12 +200,12 @@ fn replace(
         handle_kwargs("replace", kwargs, |key, value, eq| {
             if eq(key, str_year) {
                 year = value
-                    .cast::<PyInt>()
+                    .cast_allow_subclass::<PyInt>()
                     .ok_or_type_err("year must be an integer")?
                     .to_long()?;
             } else if eq(key, str_month) {
                 month = value
-                    .cast::<PyInt>()
+                    .cast_allow_subclass::<PyInt>()
                     .ok_or_type_err("month must be an integer")?
                     .to_long()?;
             } else {
@@ -222,7 +222,7 @@ fn replace(
 fn on_day(cls: HeapType<YearMonth>, slf: YearMonth, arg: PyObj) -> PyReturn {
     let YearMonth { year, month } = slf;
     let day = arg
-        .cast::<PyInt>()
+        .cast_allow_subclass::<PyInt>()
         .ok_or_type_err("day must be an integer")?
         .to_long()?
         .try_into()
@@ -254,7 +254,7 @@ static mut METHODS: &[PyMethodDef] = &[
 
 pub(crate) fn unpickle(state: &State, arg: PyObj) -> PyReturn {
     let py_bytes = arg
-        .cast::<PyBytes>()
+        .cast_exact::<PyBytes>()
         .ok_or_type_err("Invalid pickle data")?;
     let mut packed = py_bytes.as_bytes()?;
     if packed.len() != 3 {
